@@ -5,26 +5,8 @@ use base qw/HTTP::MobileAttribute::Plugin/;
 use HTTP::MobileAttribute;
 use HTTP::MobileAttribute::Plugin::Display::DoCoMoMap qw/$DisplayMap/;
 
-my %ARG_DISPATCH = (
-    ThirdForce => \&__thirdforce_args,
-    EZweb      => \&__ezweb_args,
-    DoCoMo     => \&__docomo_args,
-);
-
-sub display : Method {
+sub thirdforce :CarrierMethod('ThirdForce', 'display') {
     my ($self, $c) = @_;
-
-    my $code = $ARG_DISPATCH{$c->carrier_longname};
-    if ($code) {
-        HTTP::MobileAttribute::Plugin::Display::Display->new( $code->($c) );
-    } else {
-        return;
-    }
-}
-
-sub __thirdforce_args
-{
-    my $c = shift;
     my $request = $c->request;
 
     my($width, $height) = split /\*/, $request->get('x-jphone-display');
@@ -34,35 +16,33 @@ sub __thirdforce_args
         ($color, $depth) = $c_str =~ /^([CG])(\d+)$/;
     }
 
-    +{
+    return HTTP::MobileAttribute::Plugin::Display::Display->new(+{
         width  => $width,
         height => $height,
         color  => $color eq 'C',
         depth  => $depth,
-    };
+    });
 }
 
-sub __ezweb_args
-{
-    my $c = shift;
+sub ezweb :CarrierMethod('EZweb', 'display') {
+    my ($self, $c) = @_;
     my $request = $c->request;
 
     my ( $width, $height ) = split /,/, $request->get('x-up-devcap-screenpixels');
     my $depth = ( split /,/, $request->get('x-up-devcap-screendepth') )[0];
     my $color = $request->get('x-up-devcap-iscolor');
 
-    +{
+    return HTTP::MobileAttribute::Plugin::Display::Display->new(+{
         width  => $width,
         height => $height,
         color  => ( defined $color && $color eq '1' ),
         depth  => 2**$depth,
-    };
+    });
 }
 
-sub __docomo_args
-{
-    my $c = shift;
-    $DisplayMap->{ uc( $c->model ) };
+sub docomo :CarrierMethod('DoCoMo', 'display') {
+    my ($self, $c) = @_;
+    return HTTP::MobileAttribute::Plugin::Display::Display->new($DisplayMap->{ uc( $c->model ) });
 }
 
 package HTTP::MobileAttribute::Plugin::Display::Display;
